PROGRAM  FUTTER
 
  USE PRECISION   , ONLY:                                              &
                          PP, SP
  USE QYLSM       , ONLY:                                              &
                          FRJ, FRK, Q, A, V, W, X, Y, SIG, CHISQ
  USE NR          , ONLY:                                              &
                          SVDFIT

  IMPLICIT NONE

  INTEGER                      :: J, KF, NFREQ
  REAL (KIND=PP), PARAMETER    :: PI =3.1415926535897932384626433832795
  REAL (KIND=PP)               :: VEL, VELUN, QI, FR, FRANGE, FR1,     &
                                  R, THE1, THE2, POFRJ, POFRJ2P1,      &
                                  VELUN1, FRMAX
  REAL (KIND=PP), DIMENSION(:), ALLOCATABLE :: YMU
  REAL (KIND=PP), DIMENSION(:), ALLOCATABLE :: PLO, HEQ

INTERFACE

  FUNCTION F (X,N)
  USE nrtype
  USE QYLSM , ONLY: FRJ, FRK, Q
  IMPLICIT NONE
  REAL   (SP) , INTENT(IN)   :: x
  INTEGER(I4B), INTENT(IN)   :: n
  REAL   (SP) , DIMENSION(n) :: F
  END FUNCTION F

END INTERFACE


  WRITE(*,'(A)'      ) ' Reference frequency :'
  READ (*,  *        )  FR
  FR = FR*2._PP*PI
  WRITE(*,'(A)'      ) ' Velocity on ref. frequency :'
  READ (*,  *        )  VEL
  WRITE(*,'(A)'      ) ' Q        on ref. frequency :'
  READ (*,  *        )  QI
  WRITE(*,'(A)'      ) ' Frequency range :'
  READ (*,  *        )  FRANGE
  WRITE(*,'(A)'      ) ' Maximum frequency :'
  READ (*,  *        )  FRMAX
  WRITE(*,'(A)'      ) ' Number of rel. frequencies :'
  READ (*,  *        )  NFREQ
  FRMAX = FRMAX*2._PP*PI

  ALLOCATE (FRJ(NFREQ), YMU(NFREQ), PLO(2*NFREQ-1), HEQ(2*NFREQ-1),    &
            A(NFREQ), W(NFREQ), Q(2*NFREQ-1), X(2*NFREQ-1),            &
            SIG(2*NFREQ-1), Y(2*NFREQ-1), FRK(2*NFREQ-1),              &
            V(NFREQ,NFREQ) )
  
  DO J = 1, NFREQ
    FRJ(J)=10**(LOG10(FRMAX)-(2.*REAL(J)-1.)/2./REAL(NFREQ)*FRANGE)
  END DO

  DO J = 1, 2*NFREQ-1
    FRK(J)=10**(LOG10(FRMAX)-(REAL(J))/(REAL(NFREQ)*2.)*FRANGE)
  END DO

  DO KF=1,2*NFREQ-1
    PLO(KF)=( LOG (FRK(KF)/FR) )/PI
  END DO

!______ LAYER ___________________

  DO KF=1, 2*NFREQ-1
    X(KF)  = REAL(KF)
    HEQ(KF)= QI-PLO(KF)
  END DO

  Q   = HEQ
  Y   = 1._PP/Q
  SIG = 1._PP
  CALL SVDFIT (x,y,sig,a,v,w,chisq,F)
  YMU = A

  THE1 = 0._PP
  THE2 = 0._PP

  DO  J = 1, NFREQ

    POFRJ    = FR / FRJ(J)
    POFRJ2P1 = POFRJ * POFRJ + 1._PP

    THE1   = THE1 + YMU(J)      /POFRJ2P1
    THE2   = THE2 + YMU(J)*POFRJ/POFRJ2P1  

  END DO

  THE1  = 1._PP - THE1

  R     = SQRT( THE1*THE1 + THE2*THE2 )

  VELUN = VEL*SQRT((R+THE1)/(2._PP*R*R))

  WRITE(*,*) 'Unrelaxed velocity:', VELUN

  OPEN (10, FILE='FUTTER.DAT')
  DO KF = -50, 150
    FR1 = FRMAX/(10.**(FRANGE*REAL(100-KF,PP)/99._PP))
    THE1 = 0._PP
    THE2 = 0._PP

    DO  J = 1, NFREQ

      POFRJ    = FR1 / FRJ(J)
      POFRJ2P1 = POFRJ * POFRJ + 1._PP

      THE1   = THE1 + YMU(J)      /POFRJ2P1
      THE2   = THE2 + YMU(J)*POFRJ/POFRJ2P1  

    END DO

    THE1  = 1._PP - THE1
    R     = SQRT( THE1*THE1 + THE2*THE2 )

    VELUN1 = VELUN/SQRT((R+THE1)/(2._PP*R*R))

    ! Write:  frequency, Q approx., Q theor., V approx., V theor.
    WRITE (10,'(5(1X,F15.7))') FR1/2._PP/PI, THE1/THE2, QI-( LOG (FR1/FR) )/PI, VELUN1, &
                 VEL/(1.-1./(PI*QI)*LOG (FR1/FR))
  END DO
  CLOSE (10)


END PROGRAM  FUTTER
